package edu.vub.at.simonsays;
/*
  _________       _____  __                                  .____                                                               .____          ___.    
 /   _____/ _____/ ____\/  |___  _  _______ _______   ____   |    |   _____    ____    ____  __ _______     ____   ____   ______ |    |   _____ \_ |__  
 \_____  \ /  _ \   __\\   __\ \/ \/ /\__  \\_  __ \_/ __ \  |    |   \__  \  /    \  / ___\|  |  \__  \   / ___\_/ __ \ /  ___/ |    |   \__  \ | __ \ 
 /        (  <_> )  |   |  |  \     /  / __ \|  | \/\  ___/  |    |___ / __ \|   |  \/ /_/  >  |  // __ \_/ /_/  >  ___/ \___ \  |    |___ / __ \| \_\ \
/_______  /\____/|__|   |__|   \/\_/  (____  /__|    \___  > |_______ (____  /___|  /\___  /|____/(____  /\___  / \___  >____  > |_______ (____  /___  /
        \/                                 \/            \/          \/    \/     \//_____/            \//_____/      \/     \/          \/    \/    \/ 
      
      Assignment Simon Says
*/

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.os.Bundle;
import android.view.Menu;
import android.widget.Button;
import android.widget.GridView;
import edu.vub.at.simonsays.interfaces.ButtonListener;
import edu.vub.at.simonsays.interfaces.SimonGUI;


/* 
 * This activity is started when the application is started.  
 */

public class SimonActivity extends Activity implements ButtonListener, SimonGUI {
	//The colors which are used for the buttons. 
	public static int[] simon_colors ={ Color.GREEN, Color.RED, Color.YELLOW, Color.BLUE}; 
	//The view in which the buttons are kept.
	public GridView gridview;
	//The current sequence of "simon".
	private static Sequence color_sequence;
	//The model of the gridview. Fills the view with buttons.
	private ButtonAdapter ba;

	//Constants for communication with other activities
	private static int WRONG = 0; // requests code
	private static int CORRECT = 1; // request code
	public  static String MESSAGE = "msg"; // extra data in the intent
	public static String LEVEL = "level"; //extra data in the intent

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_simon);
		gridview = (GridView) findViewById(R.id.gridview);
		ba = new ButtonAdapter(this,this);
		gridview.setAdapter(ba);

		if (color_sequence == null) {
			//Initialize the sequence
			color_sequence = new Sequence();
			//start the game straight away
			nextRound();
		} else {
			color_sequence.play(ba, this);
		}
	}

	private void nextRound() {
		color_sequence.extend();
		//play the sequence
		color_sequence.play(ba,this);
	}


	/*
	 * Takes a button and changes the color. 
	 * Could be abstracted with an interface
	 * TODO
	 */
	public void highlightButton(final Button b, final Integer color) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				b.getBackground().setColorFilter( null );
				b.getBackground().setColorFilter(simon_colors[color], PorterDuff.Mode.DARKEN);
			}
		});
	}

	/*
	 * Takes the button and changes the color to be less highlighted 
	 * Could be abstracted with an interface
	 * TODO
	 */
	public void normalButton(final Button b, final Integer color) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				b.getBackground().setColorFilter( null );
				b.getBackground().setColorFilter(simon_colors[color], PorterDuff.Mode.SCREEN);
			}
		});
	}


	/*
	 * Inflates the menu for this activity
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_simon, menu);
		return true;
	}


	/*
	 * Get back the result of starting the result activity.
	 * requestCode	The integer request code originally supplied to startActivityForResult(), 
	 * 				allowing you to identify who this result came from.
	 * resultCode	The integer result code returned by the child activity through its setResult().
	 * data			An Intent, which can return result data to the caller (various data can be attached to Intent "extras").
	 */
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == CORRECT) {
			nextRound();
		} else {
			if  (requestCode == WRONG){
				color_sequence = new Sequence();
				color_sequence.extend();
			}
		}
	}


	/* 
	 * This is a callback function that is invoked 
	 * whenever the user presses a button.
	 * The first argument is a number that indicates 
	 * which button is pressed. 
	 */
	public void buttonPressed(int number) {
		if(color_sequence.match(number)) {
			//Correct !
			if( color_sequence.isEnd() ) {
				//You finished the current sequence !
				Intent intent = new Intent(SimonActivity.this, ResultActivity.class);
				String message = getResources().getString(R.string.correct_message) + "\n"+ getResources().getString(R.string.level_reached);
				intent.putExtra(MESSAGE, message);
				//Add the level
				startActivityForResult(intent, CORRECT);
			}
		}
		//You lost 
		else {
			//Make a new intent 
			//Build a message 
			//put the message in the intent see strings.xml (res/values/strings.xml) add a wrong message.
			//put the level in the intent 
			//start the activity for a WRONG sequence
		}
	}


}
